#! /bin/env python

"""Command-line tool for viewing hunting horn melodies.

Usage:
    horn list [--notes=<notes>] [(--horn=<horn>|--select-horn)]
    horn -h | --help
    horn --version

Options:
    --help -h        Show this screen.
    --version        Show version.
    --notes=<notes>  Notes to filter by [default: ABGMRWY]
"""

__version__ = "0.2"

from typing import Optional, Set

from docopt import docopt

from huntinghorn.constants import Note, horns, melodies
from huntinghorn.ui import prompt_for_horn
from huntinghorn.utils import Horn


def main():
    """Entry point for the command line interface."""
    arguments = docopt(__doc__, version=f"horn {__version__}")

    notes: Set[Note] = set()
    horn: Optional[Horn] = None
    note_names = dict(
        A=Note.AQUA,
        B=Note.BLUE,
        G=Note.GREEN,
        M=Note.MAGENTA,
        R=Note.RED,
        W=Note.WHITE,
        Y=Note.YELLOW
    )

    for name in arguments["--notes"]:
        if note := note_names.get(name):
            notes.add(note)

    horn = horns.get(arguments["--horn"])
    if arguments["--select-horn"]:
        horn = prompt_for_horn() or horn

    available_melodies = horn.melodies if horn else [
        melody for melody in melodies if notes >= set(melody.actions)
    ]

    padding = max(len(str(melody)) for melody in available_melodies)

    for melody in available_melodies:
        print(melody.as_str(padding + 1))



from dataclasses import dataclass
from enum import Enum
from functools import cached_property
from typing import Dict, List, Union

from colorama import Back, Fore, Style

import huntinghorn


class Button:
    UP = "▲"
    LEFT = "◀"
    DOWN = "▼"
    RIGHT = "▶"


class Note(Enum):
    RED = "RED"
    BLUE = "BLUE"
    GREEN = "GREEN"
    YELLOW = "YELLOW"
    MAGENTA = "MAGENTA"
    AQUA = "LIGHTBLUE_EX"
    WHITE = "LIGHTWHITE_EX"

    @cached_property
    def fore_colour(self) -> str:
        return getattr(Fore, self.value)

    @cached_property
    def back_colour(self) -> str:
        return getattr(Back, self.value)

    def __str__(self):
        return f"{Style.RESET_ALL}{self.fore_colour}♪{Style.RESET_ALL}"


@dataclass
class Move:
    buttons: List[str]
    note: Note

    def __str__(self) -> str:
        return (
            f"{Back.BLACK}{self.note.fore_colour}"
            f"{' '.join(self.buttons)} {Style.RESET_ALL}"
        ).ljust(18, " ")


@dataclass
class Melody:
    name: str
    actions: Union[List[Note], List[Move]]

    rest_string: str = "𝄽"

    def __post_init__(self) -> None:
        # Don't include rests for normal notes
        if type(self.actions[0]) is Note:
            self.rest_string = ""

    def __str__(self) -> str:
        return self.name

    def as_str(self, padding: int) -> str:
        return (
            f"{Fore.WHITE}{self.name.ljust(padding)}{Style.RESET_ALL}"
            f"{self.rest_string} ".join(map(str, self.actions))
        )


@dataclass
class Horn:
    """Represent an instrument that can play melodies."""

    name: str
    moves: List[Move]

    @cached_property
    def notes(self) -> Dict[Note, Move]:
        return {move.note: move for move in self.moves}

    @cached_property
    def melodies(self) -> List[Melody]:
        return [
            Melody(
                name=melody.name,
                actions=[self.notes[note] for note in melody.actions]
            )
            for melody in huntinghorn.constants.melodies
            if self.notes.keys() >= set(melody.actions)
        ]



from typing import Optional

from prompt_toolkit import completion, key_binding, prompt
from prompt_toolkit.filters import completion_is_selected, has_completions

from huntinghorn.constants import horns
from huntinghorn.utils import Horn


key_bindings = key_binding.KeyBindings()
completer = completion.FuzzyWordCompleter(horns.keys())


@key_bindings.add("enter", filter=has_completions & ~completion_is_selected)
def validate_text(event: key_binding.key_processor.KeyPressEvent) -> None:
    """Validate the current selection once enter is pressed."""
    event.current_buffer.go_to_completion(0)
    event.current_buffer.validate_and_handle()


def prompt_for_horn() -> Optional[Horn]:
    """Prompt the user to select a Horn."""
    name = str(prompt(
        "Horn: ",
        completer=completer,
        complete_while_typing=True,
        key_bindings=key_bindings,
    ))
    print()
    return horns.get(name)



"""Constants and Enums for use in the main program."""

from typing import Dict, List

from huntinghorn.utils import Button, Horn, Melody, Move, Note


horns: Dict[str, Horn] = {
    "Volcanic Rock": Horn("Volcanic Rock", [
        Move([Button.RIGHT], Note.MAGENTA),
        Move([Button.UP, Button.RIGHT], Note.AQUA),
        Move([Button.UP], Note.YELLOW)
    ])
}

melodies: List[Melody] = [
    Melody("Speed up", [Note.WHITE, Note.WHITE]),
    Melody("Speed up", [Note.MAGENTA, Note.MAGENTA]),
    Melody("Clairvoyance", [Note.BLUE, Note.BLUE, Note.AQUA]),
    Melody("Supersonic Waves", [Note.YELLOW, Note.YELLOW, Note.YELLOW]),
    Melody("Attack Up [Lo]", [Note.WHITE, Note.RED]),
    Melody("Attack Up [Hi]", [Note.MAGENTA, Note.RED, Note.RED]),
    Melody("Defense Up [Lo]", [Note.RED, Note.YELLOW]),
    Melody("Defense Up [Hi]", [Note.RED, Note.GREEN, Note.RED]),
    Melody("Defense Up [Hi]", [Note.RED, Note.AQUA, Note.AQUA, Note.MAGENTA]),
    Melody("Health Inc [Lo]", [Note.RED, Note.BLUE, Note.WHITE]),
    Melody("Health Inc [Mid]", [Note.RED, Note.RED, Note.AQUA]),
    Melody("Health Inc [Hi]", [Note.RED, Note.BLUE, Note.RED, Note.MAGENTA]),
    Melody("Wind Reduce", [Note.BLUE, Note.BLUE, Note.RED]),
    Melody("Wind Cancel", [Note.BLUE, Note.BLUE, Note.GREEN]),
    Melody("Wind Cancel", [Note.BLUE, Note.AQUA, Note.BLUE]),
    Melody("Wind All Cancel", [Note.BLUE, Note.BLUE, Note.YELLOW, Note.MAGENTA]),
    Melody("Marathon [Lo]", [Note.WHITE, Note.BLUE]),
    Melody("Marathon [Hi]", [Note.MAGENTA, Note.BLUE, Note.BLUE]),
    Melody("Health Rec [Lo]", [Note.MAGENTA, Note.GREEN]),
    Melody("Health Rec [Mid]", [Note.GREEN, Note.MAGENTA, Note.YELLOW]),
    Melody("Health Rec [Hi]", [Note.GREEN, Note.GREEN, Note.MAGENTA, Note.AQUA]),
    Melody("Health Rec [Lo] & Antidote", [Note.WHITE, Note.GREEN]),
    Melody("Health Rec [Mid] & Antidote", [Note.GREEN, Note.BLUE, Note.MAGENTA, Note.BLUE]),
    Melody("Health Rec [Mid] & Deodorant", [Note.GREEN, Note.WHITE, Note.AQUA]),
    Melody("Recover Spd Up [Lo]", [Note.GREEN, Note.GREEN, Note.YELLOW]),
    Melody("Recover Spd Up [Hi]", [Note.GREEN, Note.RED, Note.GREEN, Note.MAGENTA]),
    Melody("Heavenly Protection", [Note.GREEN, Note.YELLOW, Note.MAGENTA, Note.YELLOW]),
    Melody("Hear Protect [Lo]", [Note.AQUA, Note.AQUA, Note.RED]),
    Melody("Hear Protect [Hi]", [Note.AQUA, Note.AQUA, Note.GREEN, Note.MAGENTA]),
    Melody("No Cold & Snow Res", [Note.AQUA, Note.AQUA, Note.YELLOW]),
    Melody("No Heat", [Note.AQUA, Note.GREEN, Note.AQUA]),
    Melody("No Faint", [Note.AQUA, Note.BLUE, Note.MAGENTA]),
    Melody("No Paralysis", [Note.AQUA, Note.YELLOW, Note.YELLOW]),
    Melody("No Quakes", [Note.AQUA, Note.YELLOW, Note.AQUA]),
    Melody("Element Attack Up", [Note.YELLOW, Note.BLUE, Note.YELLOW, Note.WHITE]),
    Melody("Fire Res Up [Lo]", [Note.YELLOW, Note.RED]),
    Melody("Fire Res Up [Hi]", [Note.YELLOW, Note.BLUE, Note.WHITE]),
    Melody("Water Res Up [Lo]", [Note.YELLOW, Note.BLUE, Note.MAGENTA]),
    Melody("Water Res Up [Hi]", [Note.YELLOW, Note.BLUE, Note.BLUE, Note.WHITE]),
    Melody("Thunder Res Up [Lo]", [Note.YELLOW, Note.AQUA, Note.MAGENTA]),
    Melody("Thunder Res Up [Hi]", [Note.YELLOW, Note.YELLOW, Note.BLUE]),
    Melody("Ice Res Up [Lo]", [Note.YELLOW, Note.WHITE, Note.AQUA]),
    Melody("Ice Res Up [Lo]", [Note.YELLOW, Note.GREEN, Note.WHITE]),
    Melody("Ice Res Up [Hi]", [Note.YELLOW, Note.MAGENTA, Note.AQUA]),
    Melody("Ice Res Up [Hi]", [Note.YELLOW, Note.GREEN, Note.MAGENTA]),
    Melody("Dragon Res Up [Lo]", [Note.WHITE, Note.YELLOW]),
    Melody("Dragon Res Up [Hi]", [Note.MAGENTA, Note.YELLOW, Note.YELLOW])
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['huntinghorn']

package_data = \
{'': ['*']}

install_requires = \
['bullet>=2.2.0,<3.0.0',
 'colorama>=0.4.3,<0.5.0',
 'docopt>=0.6.2,<0.7.0',
 'prompt-toolkit>=3.0.7,<4.0.0']

entry_points = \
{'console_scripts': ['horn = huntinghorn:main']}

setup_kwargs = {
    'name': 'huntinghorn',
    'version': '0.2',
    'description': 'Command-Line tool for Hunting Horn users.',
    'long_description': None,
    'author': 'Jeremiah Boby',
    'author_email': 'mail@jeremiahboby.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
